@@ -102,10 +102,15 @@ def compute_index(index: ComplexityIndex, element: TextElement) -> float:
     return index.process(element)
 
 # computed indices and saves for each TextElement in indices dictionary
-def compute_indices(doc: Document, cna_graph: CnaGraph = None):
+def compute_indices(doc: Document, cna_graph: CnaGraph = None, parallel = True):
     logger.info('Starting computing all indices for {0} type element'.format(type(doc).__name__))
     download_wordlist(doc.lang)
     num_cores = cpu_count()
-    Parallel(n_jobs=num_cores, prefer="threads")(delayed(compute_index)(index, doc) \
-        for cat in IndexCategory for index in cat.create(doc.lang, cna_graph))
+    if parallel:
+        Parallel(n_jobs=num_cores, prefer="threads")(delayed(compute_index)(index, doc) \
+            for cat in IndexCategory for index in cat.create(doc.lang, cna_graph))
+    else:
+        for cat in IndexCategory:
+            for index in cat.create(doc.lang, cna_graph):
+                compute_index(index, doc)
 
